@@ -58,8 +58,6 @@
  * SUCH DAMAGE.
  */
 
-#include <sys/cdefs.h>
-
 #include <sys/queue.h>
 #include <sys/types.h>
 #include <archive.h>
@@ -74,7 +72,7 @@
 
 #include "ar.h"
 
-ELFTC_VCSID("$Id: ar.c 2485 2012-04-07 15:54:59Z kaiwang27 $");
+ELFTC_VCSID("$Id: ar.c 3319 2016-01-13 21:37:53Z jkoshy $");
 
 enum options
 {
@@ -119,13 +117,13 @@ main(int argc, char **argv)
 
 	/*
 	 * Act like ranlib if our name ends in "ranlib"; this
-	 * accomodates names like "arm-freebsd7.1-ranlib",
+	 * accommodates names like "arm-freebsd7.1-ranlib",
 	 * "bsdranlib", etc.
 	 */
 	len = strlen(bsdar->progname);
 	if (len >= strlen("ranlib") &&
 	    strcmp(bsdar->progname + len - strlen("ranlib"), "ranlib") == 0) {
-		while ((opt = getopt_long(argc, argv, "tDV", longopts,
+		while ((opt = getopt_long(argc, argv, "tDUV", longopts,
 		    NULL)) != -1) {
 			switch(opt) {
 			case 't':
@@ -134,6 +132,9 @@ main(int argc, char **argv)
 			case 'D':
 				bsdar->options |= AR_D;
 				break;
+			case 'U':
+				bsdar->options &= ~AR_D;
+				break;
 			case 'V':
 				bsdar_version();
 				break;
@@ -150,7 +151,7 @@ main(int argc, char **argv)
 			ranlib_usage();
 
 		bsdar->options |= AR_S;
-		for (;(bsdar->filename = *argv++) != NULL;)
+		while ((bsdar->filename = *argv++) != NULL)
 			ar_write_archive(bsdar, 's');
 
 		exit(EXIT_SUCCESS);
@@ -171,7 +172,7 @@ main(int argc, char **argv)
 		}
 	}
 
-	while ((opt = getopt_long(argc, argv, "abCcdDfF:ijlMmopqrSsTtuVvxz",
+	while ((opt = getopt_long(argc, argv, "abCcdDfF:ijlMmopqrSsTtUuVvxz",
 	    longopts, NULL)) != -1) {
 		switch(opt) {
 		case 'a':
@@ -239,6 +240,9 @@ main(int argc, char **argv)
 		case 't':
 			set_mode(bsdar, opt);
 			break;
+		case 'U':
+			bsdar->options &= ~AR_D;
+			break;
 		case 'u':
 			bsdar->options |= AR_U;
 			break;
@@ -402,7 +406,8 @@ Usage: %s <command> [options] archive file...\n\
   -D            Use fixed metadata, for consistent archive checksums.\n\
   -F FORMAT | --flavor=FORMAT\n\
                 Create archives with the specified format.\n\
-  -S            Do not generate an archive symbol table.\n"
+  -S            Do not generate an archive symbol table.\n\
+  -U            Use original metadata for archive members.\n"
 
 static void
 bsdar_usage(void)
@@ -417,6 +422,7 @@ Usage: %s [options] archive...\n\
   Options:\n\
   -t              (This option is accepted, but ignored).\n\
   -D              Use fixed metadata, for consistent archive checksums.\n\
+  -U              Use original metadata, for unique archive checksums.\n\
   -V              Print a version identifier and exit.\n"
 
 static void
