@@ -25,7 +25,6 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <sys/cdefs.h>
 #include <sys/queue.h>
 #include <sys/stat.h>
 
@@ -39,7 +38,7 @@
 
 #include "ar.h"
 
-ELFTC_VCSID("$Id: read.c 2229 2011-11-27 13:25:37Z jkoshy $");
+ELFTC_VCSID("$Id: read.c 3180 2015-04-09 15:13:57Z emaste $");
 
 /*
  * Handle read modes: 'x', 't' and 'p'.
@@ -68,9 +67,8 @@ ar_read_archive(struct bsdar *bsdar, int mode)
 
 	if ((a = archive_read_new()) == NULL)
 		bsdar_errc(bsdar, 0, "archive_read_new failed");
-	archive_read_support_compression_none(a);
 	archive_read_support_format_ar(a);
-	AC(archive_read_open_file(a, bsdar->filename, DEF_BLKSZ));
+	AC(archive_read_open_filename(a, bsdar->filename, DEF_BLKSZ));
 
 	out = bsdar->output;
 
@@ -91,7 +89,8 @@ ar_read_archive(struct bsdar *bsdar, int mode)
 		else
 			bsdar->options &= ~AR_BSD;
 
-		name = archive_entry_pathname(entry);
+		if ((name = archive_entry_pathname(entry)) == NULL)
+			break;
 
 		/* Skip pseudo members. */
 		if (bsdar_is_pseudomember(bsdar, name))
@@ -175,7 +174,15 @@ ar_read_archive(struct bsdar *bsdar, int mode)
 
 				if (bsdar->options & AR_V)
 					(void)fprintf(out, "x - %s\n", name);
-				flags = 0;
+				/* Disallow absolute paths. */
+				if (name[0] == '/') {
+					bsdar_warnc(bsdar, 0,
+					    "Absolute path '%s'", name);
+					continue;
+				}
+				/* Basic path security flags. */
+				flags = ARCHIVE_EXTRACT_SECURE_SYMLINKS |
+	 			    ARCHIVE_EXTRACT_SECURE_NODOTDOT;
 				if (bsdar->options & AR_O)
 					flags |= ARCHIVE_EXTRACT_TIME;
 
@@ -188,5 +195,5 @@ ar_read_archive(struct bsdar *bsdar, int mode)
 		}
 	}
 	AC(archive_read_close(a));
-	ACV(archive_read_finish(a));
+	ACV(archive_read_free(a));
 }
