@@ -24,7 +24,6 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <sys/cdefs.h>
 #include <sys/queue.h>
 #include <sys/stat.h>
 
@@ -41,7 +40,7 @@
 
 #include "ar.h"
 
-ELFTC_VCSID("$Id: write.c 2496 2012-04-24 02:33:40Z jkoshy $");
+ELFTC_VCSID("$Id: write.c 3183 2015-04-10 16:18:42Z emaste $");
 
 #define _ARMAG_LEN 8		/* length of the magic string */
 #define _ARHDR_LEN 60		/* length of the archive header */
@@ -70,7 +69,7 @@ static void	write_objs(struct bsdar *bsdar);
 /*
  * Create an object from a file, and return the created object
  * descriptor.  Return NULL if either an error occurs, or if the '-u'
- * option was specifed and the member is not newer than the existing
+ * option was specified and the member is not newer than the existing
  * one in the archive.
  */
 static struct ar_obj *
@@ -259,7 +258,6 @@ read_objs(struct bsdar *bsdar, const char *archive, int checkargv)
 
 	if ((a = archive_read_new()) == NULL)
 		bsdar_errc(bsdar, 0, "archive_read_new failed");
-	archive_read_support_compression_none(a);
 	archive_read_support_format_ar(a);
 	AC(archive_read_open_filename(a, archive, DEF_BLKSZ));
 	for (;;) {
@@ -349,7 +347,7 @@ read_objs(struct bsdar *bsdar, const char *archive, int checkargv)
 		TAILQ_INSERT_TAIL(&bsdar->v_obj, obj, objs);
 	}
 	AC(archive_read_close(a));
-	ACV(archive_read_finish(a));
+	ACV(archive_read_free(a));
 }
 
 /*
@@ -428,7 +426,7 @@ ar_write_archive(struct bsdar *bsdar, int mode)
 	if (mode == 'A') {
 		/*
 		 * Read objects from the target archive of the
-		 * 'ADDLIB' command.  If there are members spcified in
+		 * 'ADDLIB' command.  If there are members specified in
 		 * 'argv', read those members only, otherwise the
 		 * entire archive will be read.
 		 */
@@ -449,7 +447,7 @@ ar_write_archive(struct bsdar *bsdar, int mode)
 
 		/*
 		 * If we cannot find the position specified by the
-		 * user, sliently insert objects at the tail of the
+		 * user, silently insert objects at the tail of the
 		 * list.
 		 */
 		if (pos == NULL)
@@ -732,7 +730,6 @@ write_objs(struct bsdar *bsdar)
 		archive_write_set_format_ar_bsd(a);
 	else
 		archive_write_set_format_ar_svr4(a);
-	archive_write_set_compression_none(a);
 
 	AC(archive_write_open_filename(a, bsdar->filename));
 
@@ -795,7 +792,7 @@ write_objs(struct bsdar *bsdar)
 	}
 
 	AC(archive_write_close(a));
-	ACV(archive_write_finish(a));
+	ACV(archive_write_free(a));
 }
 
 /*
