@@ -1,7 +1,7 @@
 /*-
  * Copyright (c) 2008 Hyogeol Lee
  * Copyright (c) 2000, 2001 David O'Brien
- * Copyright (c) 1996 Sren Schmidt
+ * Copyright (c) 1996 Søren Schmidt
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -28,8 +28,6 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <sys/cdefs.h>
-
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <err.h>
@@ -46,7 +44,7 @@
 
 #include "_elftc.h"
 
-ELFTC_VCSID("$Id: brandelf.c 2324 2011-12-12 06:36:27Z jkoshy $");
+ELFTC_VCSID("$Id: brandelf.c 3354 2016-01-18 21:50:15Z jkoshy $");
 
 static int elftype(const char *);
 static const char *iselftype(int);
@@ -96,7 +94,7 @@ main(int argc, char **argv)
 	Elf_Kind kind;
 	int type = ELFOSABI_NONE;
 	int retval = 0;
-	int ch, change = 0, verbose = 0, force = 0, listed = 0;
+	int ch, change = 0, force = 0, listed = 0;
 
 	if (elf_version(EV_CURRENT) == EV_NONE)
 		errx(EXIT_FAILURE, "elf_version error");
@@ -124,7 +122,7 @@ main(int argc, char **argv)
 			listed = 1;
 			break;
 		case 'v':
-			verbose = 1;
+			/* This flag is ignored. */
 			break;
 		case 't':
 			if (force)
@@ -214,7 +212,7 @@ main(int argc, char **argv)
 			/*
 			 * Update the ABI type.
 			 */
-			ehdr.e_ident[EI_OSABI] = type;
+			ehdr.e_ident[EI_OSABI] = (unsigned char) type;
 			if (gelf_update_ehdr(elf, &ehdr) == 0) {
 				warnx("gelf_update_ehdr error: %s",
 				    elf_errmsg(-1));
@@ -256,7 +254,6 @@ Usage: %s [options] file...\n\
   -h | --help               Print a usage message and exit.\n\
   -l                        List known ELF ABI names.\n\
   -t ABI                    Set the ELF ABI to the value named by \"ABI\".\n\
-  -v                        Be verbose.\n\
   -V | --version            Print a version identifier and exit.\n"
 
 static void
