@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: _elftc.h 2495 2012-04-23 05:31:56Z jkoshy $
+ * $Id: _elftc.h 3244 2015-08-31 19:53:08Z emaste $
  */
 
 /**
@@ -76,6 +76,20 @@
  * SUCH DAMAGE.
  */
 
+#ifndef	LIST_FOREACH_SAFE
+#define	LIST_FOREACH_SAFE(var, head, field, tvar)		\
+	for ((var) = LIST_FIRST((head));			\
+	    (var) && ((tvar) = LIST_NEXT((var), field), 1);	\
+	    (var) = (tvar))
+#endif
+
+#ifndef	SLIST_FOREACH_SAFE
+#define	SLIST_FOREACH_SAFE(var, head, field, tvar)		\
+	for ((var) = SLIST_FIRST((head));			\
+	    (var) && ((tvar) = SLIST_NEXT((var), field), 1);	\
+	    (var) = (tvar))
+#endif
+
 #ifndef	STAILQ_CONCAT
 #define	STAILQ_CONCAT(head1, head2) do {			\
 	if (!STAILQ_EMPTY((head2))) {				\
@@ -193,6 +207,70 @@ struct name {							\
 } while (/*CONSTCOND*/0)
 #endif
 
+/*
+ * The STAILQ_SORT macro is adapted from Simon Tatham's O(n*log(n))
+ * mergesort algorithm.
+ */
+#ifndef	STAILQ_SORT
+#define	STAILQ_SORT(head, type, field, cmp) do {			\
+	STAILQ_HEAD(, type) _la, _lb;					\
+	struct type *_p, *_q, *_e;					\
+	int _i, _sz, _nmerges, _psz, _qsz;				\
+									\
+	_sz = 1;							\
+	do {								\
+		_nmerges = 0;						\
+		STAILQ_INIT(&_lb);					\
+		while (!STAILQ_EMPTY((head))) {				\
+			_nmerges++;					\
+			STAILQ_INIT(&_la);				\
+			_psz = 0;					\
+			for (_i = 0; _i < _sz && !STAILQ_EMPTY((head));	\
+			     _i++) {					\
+				_e = STAILQ_FIRST((head));		\
+				if (_e == NULL)				\
+					break;				\
+				_psz++;					\
+				STAILQ_REMOVE_HEAD((head), field);	\
+				STAILQ_INSERT_TAIL(&_la, _e, field);	\
+			}						\
+			_p = STAILQ_FIRST(&_la);			\
+			_qsz = _sz;					\
+			_q = STAILQ_FIRST((head));			\
+			while (_psz > 0 || (_qsz > 0 && _q != NULL)) {	\
+				if (_psz == 0) {			\
+					_e = _q;			\
+					_q = STAILQ_NEXT(_q, field);	\
+					STAILQ_REMOVE_HEAD((head),	\
+					    field);			\
+					_qsz--;				\
+				} else if (_qsz == 0 || _q == NULL) {	\
+					_e = _p;			\
+					_p = STAILQ_NEXT(_p, field);	\
+					STAILQ_REMOVE_HEAD(&_la, field);\
+					_psz--;				\
+				} else if (cmp(_p, _q) <= 0) {		\
+					_e = _p;			\
+					_p = STAILQ_NEXT(_p, field);	\
+					STAILQ_REMOVE_HEAD(&_la, field);\
+					_psz--;				\
+				} else {				\
+					_e = _q;			\
+					_q = STAILQ_NEXT(_q, field);	\
+					STAILQ_REMOVE_HEAD((head),	\
+					    field);			\
+					_qsz--;				\
+				}					\
+				STAILQ_INSERT_TAIL(&_lb, _e, field);	\
+			}						\
+		}							\
+		(head)->stqh_first = _lb.stqh_first;			\
+		(head)->stqh_last = _lb.stqh_last;			\
+		_sz *= 2;						\
+	} while (_nmerges > 1);						\
+} while (/*CONSTCOND*/0)
+#endif
+
 #ifndef	TAILQ_FOREACH_SAFE
 #define TAILQ_FOREACH_SAFE(var, head, field, tvar)                      \
 	for ((var) = TAILQ_FIRST((head));                               \
@@ -216,7 +294,8 @@ struct name {							\
 #define	ELFTC_VCSID(ID)		__FBSDID(ID)
 #endif
 
-#if defined(__linux__) || defined(__GNU__) || defined(__GLIBC__)
+#if defined(__APPLE__) || defined(__GLIBC__) || defined(__GNU__) || \
+    defined(__linux__)
 #if defined(__GNUC__)
 #define	ELFTC_VCSID(ID)		__asm__(".ident\t\"" ID "\"")
 #else
@@ -252,8 +331,8 @@ struct name {							\
 
 #ifndef	ELFTC_GETPROGNAME
 
-#if defined(__DragonFly__) || defined(__FreeBSD__) || defined(__minix) || \
-    defined(__NetBSD__)
+#if defined(__APPLE__) || defined(__DragonFly__) || defined(__FreeBSD__) || \
+    defined(__minix) || defined(__NetBSD__)
 
 #include <stdlib.h>
 
@@ -262,17 +341,18 @@ struct name {							\
 #endif	/* __DragonFly__ || __FreeBSD__ || __minix || __NetBSD__ */
 
 
-#if defined(__GLIBC__)
-
+#if defined(__GLIBC__) || defined(__linux__)
+#ifndef _GNU_SOURCE
 /*
  * GLIBC based systems have a global 'char *' pointer referencing
  * the executable's name.
  */
 extern const char *program_invocation_short_name;
+#endif	/* !_GNU_SOURCE */
 
 #define	ELFTC_GETPROGNAME()	program_invocation_short_name
 
-#endif	/* __GLIBC__ */
+#endif	/* __GLIBC__ || __linux__ */
 
 
 #if defined(__OpenBSD__)
@@ -290,6 +370,23 @@ extern const char *__progname;
  ** Per-OS configuration.
  **/
 
+#if defined(__APPLE__)
+
+#include <libkern/OSByteOrder.h>
+#define	htobe32(x)	OSSwapHostToBigInt32(x)
+#define	roundup2	roundup
+
+#define	ELFTC_BYTE_ORDER			_BYTE_ORDER
+#define	ELFTC_BYTE_ORDER_LITTLE_ENDIAN		_LITTLE_ENDIAN
+#define	ELFTC_BYTE_ORDER_BIG_ENDIAN		_BIG_ENDIAN
+
+#define	ELFTC_HAVE_MMAP				1
+#define	ELFTC_HAVE_STRMODE			1
+
+#define ELFTC_NEED_BYTEORDER_EXTENSIONS		1
+#endif /* __APPLE__ */
+
+
 #if defined(__DragonFly__)
 
 #include <osreldate.h>
@@ -303,7 +400,7 @@ extern const char *__progname;
 
 #endif
 
-#if defined(__GLIBC__)
+#if defined(__GLIBC__) || defined(__linux__)
 
 #include <endian.h>
 
@@ -323,7 +420,7 @@ extern const char *__progname;
 
 #define	roundup2	roundup
 
-#endif	/* __GLIBC__ */
+#endif	/* __GLIBC__ || __linux__ */
 
 
 #if defined(__FreeBSD__)
