@@ -24,7 +24,6 @@
  * SUCH DAMAGE.
  */
 
-#include <sys/cdefs.h>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <err.h>
@@ -36,7 +35,7 @@
 
 #include "elfcopy.h"
 
-ELFTC_VCSID("$Id: binary.c 2358 2011-12-19 18:22:32Z kaiwang27 $");
+ELFTC_VCSID("$Id: binary.c 3270 2015-12-11 18:48:56Z emaste $");
 
 /*
  * Convert ELF object to `binary'. Sections with SHF_ALLOC flag set
@@ -141,6 +140,7 @@ create_elf_from_binary(struct elfcopy *ecp, int ifd, const char *ifn)
 	GElf_Shdr sh;
 	void *content;
 	uint64_t off, data_start, data_end, data_size;
+	char *sym_basename, *p;
 
 	/* Reset internal section list. */
 	if (!TAILQ_EMPTY(&ecp->v_sec))
@@ -211,8 +211,13 @@ create_elf_from_binary(struct elfcopy *ecp, int ifd, const char *ifn)
 	/* Count in .symtab and .strtab section headers.  */
 	shtab->sz += gelf_fsize(ecp->eout, ELF_T_SHDR, 2, EV_CURRENT);
 
+	if ((sym_basename = strdup(ifn)) == NULL)
+		err(1, "strdup");
+	p = sym_basename;
+	while ((p = strchr(p, '.')) != NULL)
+		*p++ = '_';
 #define	_GEN_SYMNAME(S) do {						\
-	snprintf(name, sizeof(name), "%s%s%s", "_binary_", ifn, S);	\
+	snprintf(name, sizeof(name), "%s%s%s", "_binary_", sym_basename, S); \
 } while (0)
 
 	/*
@@ -234,6 +239,7 @@ create_elf_from_binary(struct elfcopy *ecp, int ifd, const char *ifn)
 	finalize_external_symtab(ecp);
 	create_symtab_data(ecp);
 #undef	_GEN_SYMNAME
+	free(sym_basename);
 
 	/*
 	 * Write the underlying ehdr. Note that it should be called
