@@ -24,17 +24,17 @@
  * SUCH DAMAGE.
  */
 
-#include <sys/cdefs.h>
 #include <sys/queue.h>
 #include <err.h>
 #include <gelf.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "elfcopy.h"
 
-ELFTC_VCSID("$Id: segments.c 2542 2012-08-12 16:14:15Z kaiwang27 $");
+ELFTC_VCSID("$Id: segments.c 3409 2016-02-14 18:52:19Z jkoshy $");
 
 static void	insert_to_inseg_list(struct segment *seg, struct section *sec);
 
@@ -72,17 +72,19 @@ add_to_inseg_list(struct elfcopy *ecp, struct section *s)
 	 */
 	loadable = 0;
 	STAILQ_FOREACH(seg, &ecp->v_seg, seg_list) {
-		if (s->off < seg->off)
+		if (s->off < seg->off || (s->vma < seg->addr && !s->pseudo))
 			continue;
 		if (s->off + s->sz > seg->off + seg->fsz &&
 		    s->type != SHT_NOBITS)
 			continue;
-		if (s->off + s->sz > seg->off + seg->msz)
+		if (s->vma + s->sz > seg->addr + seg->msz)
 			continue;
 
 		insert_to_inseg_list(seg, s);
 		if (seg->type == PT_LOAD)
 			s->seg = seg;
+		else if (seg->type == PT_TLS)
+			s->seg_tls = seg;
 		s->lma = seg->addr + (s->off - seg->off);
 		loadable = 1;
 	}
@@ -96,7 +98,7 @@ adjust_addr(struct elfcopy *ecp)
 	struct section *s, *s0;
 	struct segment *seg;
 	struct sec_action *sac;
-	uint64_t dl, lma, old_vma, start, end;
+	uint64_t dl, lma, start, end;
 	int found, i;
 
 	/*
@@ -105,16 +107,14 @@ adjust_addr(struct elfcopy *ecp)
 	TAILQ_FOREACH(s, &ecp->v_sec, sec_list) {
 
 		/* Only adjust loadable section's address. */
-		if (!s->loadable || s->seg == NULL)
+		if (!s->loadable)
 			continue;
 
 		/* Apply global LMA adjustment. */
-		if (ecp->change_addr != 0)
+		if (ecp->change_addr != 0 && s->seg != NULL)
 			s->lma += ecp->change_addr;
 
 		if (!s->pseudo) {
-			old_vma = s->vma;
-
 			/* Apply global VMA adjustment. */
 			if (ecp->change_addr != 0)
 				s->vma += ecp->change_addr;
@@ -135,7 +135,10 @@ adjust_addr(struct elfcopy *ecp)
 	 */
 	TAILQ_FOREACH(s, &ecp->v_sec, sec_list) {
 
-		/* Only adjust loadable section's LMA. */
+		/*
+		 * Only loadable section that's inside a segment can have
+		 * LMA adjusted.
+		 */
 		if (!s->loadable || s->seg == NULL)
 			continue;
 
@@ -173,7 +176,7 @@ adjust_addr(struct elfcopy *ecp)
 		if (lma % s->align != 0)
 			errx(EXIT_FAILURE, "The load address %#jx for "
 			    "section %s is not aligned to %ju",
-			    (uintmax_t) lma, s->name, s->align);
+			    (uintmax_t) lma, s->name, (uintmax_t) s->align);
 
 		if (lma < s->lma) {
 			/* Move section to lower address. */
@@ -214,7 +217,8 @@ adjust_addr(struct elfcopy *ecp)
 				continue;
 			errx(EXIT_FAILURE, "The extent of segment containing "
 			    "section %s overlaps with segment(%#jx,%#jx)",
-			    s->name, seg->addr, seg->addr + seg->msz);
+			    s->name, (uintmax_t) seg->addr,
+			    (uintmax_t) (seg->addr + seg->msz));
 		}
 
 		/*
@@ -438,9 +442,9 @@ copy_phdr(struct elfcopy *ecp)
 		seg->fsz = seg->msz = 0;
 		for (i = 0; i < seg->nsec; i++) {
 			s = seg->v_sec[i];
-			seg->msz = s->off + s->sz - seg->off;
+			seg->msz = s->vma + s->sz - seg->addr;
 			if (s->type != SHT_NOBITS)
-				seg->fsz = seg->msz;
+				seg->fsz = s->off + s->sz - seg->off;
 		}
 	}
 
@@ -485,7 +489,7 @@ copy_phdr(struct elfcopy *ecp)
 		ophdr.p_filesz = seg->fsz;
 		ophdr.p_memsz = seg->msz;
 		if (!gelf_update_phdr(ecp->eout, i, &ophdr))
-			err(EXIT_FAILURE, "gelf_update_phdr failed :%s",
+			errx(EXIT_FAILURE, "gelf_update_phdr failed: %s",
 			    elf_errmsg(-1));
 
 		i++;
