@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2007-2011 Kai Wang
+ * Copyright (c) 2007-2013 Kai Wang
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,8 +24,8 @@
  * SUCH DAMAGE.
  */
 
-#include <sys/cdefs.h>
 #include <sys/param.h>
+#include <assert.h>
 #include <err.h>
 #include <fnmatch.h>
 #include <stdio.h>
@@ -34,7 +34,7 @@
 
 #include "elfcopy.h"
 
-ELFTC_VCSID("$Id: symbols.c 2358 2011-12-19 18:22:32Z kaiwang27 $");
+ELFTC_VCSID("$Id: symbols.c 3376 2016-01-26 18:41:39Z emaste $");
 
 /* Symbol table buffer structure. */
 struct symbuf {
@@ -46,25 +46,41 @@ struct symbuf {
 	size_t gcap, lcap; 	/* buffer capacities. */
 };
 
+struct sthash {
+	LIST_ENTRY(sthash) sh_next;
+	size_t sh_off;
+};
+typedef LIST_HEAD(,sthash) hash_head;
+#define STHASHSIZE 65536
+
+struct strimpl {
+	char *buf;		/* string table */
+	size_t sz;		/* entries */
+	size_t cap;		/* buffer capacity */
+	hash_head hash[STHASHSIZE];
+};
+
+
 /* String table buffer structure. */
 struct strbuf {
-	char *l;		/* local symbol string table */
-	char *g;		/* global symbol string table */
-	size_t lsz, gsz;	/* size of each kind */
-	size_t gcap, lcap; 	/* buffer capacities. */
+	struct strimpl l;	/* local symbols */
+	struct strimpl g;	/* global symbols */
 };
 
 static int	is_debug_symbol(unsigned char st_info);
 static int	is_global_symbol(unsigned char st_info);
 static int	is_local_symbol(unsigned char st_info);
+static int	is_local_label(const char *name);
 static int	is_needed_symbol(struct elfcopy *ecp, int i, GElf_Sym *s);
 static int	is_remove_symbol(struct elfcopy *ecp, size_t sc, int i,
 		    GElf_Sym *s, const char *name);
 static int	is_weak_symbol(unsigned char st_info);
-static int	lookup_exact_string(const char *buf, size_t sz, const char *s);
+static int	lookup_exact_string(hash_head *hash, const char *buf,
+		    const char *s);
 static int	generate_symbols(struct elfcopy *ecp);
-static void	mark_symbols(struct elfcopy *ecp, size_t sc);
-static int	match_wildcard(const char *name, const char *pattern);
+static void	mark_reloc_symbols(struct elfcopy *ecp, size_t sc);
+static void	mark_section_group_symbols(struct elfcopy *ecp, size_t sc);
+uint32_t	str_hash(const char *s);
 
 /* Convenient bit vector operation macros. */
 #define BIT_SET(v, n) (v[(n)>>3] |= 1U << ((n) & 7))
@@ -86,7 +102,8 @@ static int
 is_global_symbol(unsigned char st_info)
 {
 
-	if (GELF_ST_BIND(st_info) == STB_GLOBAL)
+	if (GELF_ST_BIND(st_info) == STB_GLOBAL ||
+	    GELF_ST_BIND(st_info) == STB_GNU_UNIQUE)
 		return (1);
 
 	return (0);
@@ -112,6 +129,28 @@ is_local_symbol(unsigned char st_info)
 	return (0);
 }
 
+static int
+is_hidden_symbol(unsigned char st_other)
+{
+
+	if (GELF_ST_VISIBILITY(st_other) == STV_HIDDEN ||
+	    GELF_ST_VISIBILITY(st_other) == STV_INTERNAL)
+		return (1);
+
+	return (0);
+}
+
+static int
+is_local_label(const char *name)
+{
+
+	/* Compiler generated local symbols that start with .L */
+	if (name[0] == '.' && name[1] == 'L')
+		return (1);
+
+	return (0);
+}
+
 /*
  * Symbols related to relocation are needed.
  */
@@ -123,6 +162,10 @@ is_needed_symbol(struct elfcopy *ecp, int i, GElf_Sym *s)
 	if (BIT_ISSET(ecp->v_rel, i))
 		return (1);
 
+	/* Symbols refered by COMDAT sections are needed. */
+	if (BIT_ISSET(ecp->v_grp, i))
+		return (1);
+
 	/*
 	 * For relocatable files (.o files), global and weak symbols
 	 * are needed.
@@ -148,12 +191,6 @@ is_remove_symbol(struct elfcopy *ecp, size_t sc, int i, GElf_Sym *s,
 		SHN_UNDEF,	/* st_shndx */
 	};
 
-	if (lookup_symop_list(ecp, name, SYMOP_KEEP) != NULL)
-		return (0);
-
-	if (lookup_symop_list(ecp, name, SYMOP_STRIP) != NULL)
-		return (1);
-
 	/*
 	 * Keep the first symbol if it is the special reserved symbol.
 	 * XXX Should we generate one if it's missing?
@@ -166,11 +203,33 @@ is_remove_symbol(struct elfcopy *ecp, size_t sc, int i, GElf_Sym *s,
 	    ecp->secndx[s->st_shndx] == 0)
 		return (1);
 
+	/* Keep the symbol if specified by command line option -K. */
+	if (lookup_symop_list(ecp, name, SYMOP_KEEP) != NULL)
+		return (0);
+
 	if (ecp->strip == STRIP_ALL)
 		return (1);
 
+	/* Mark symbols used in relocation. */
 	if (ecp->v_rel == NULL)
-		mark_symbols(ecp, sc);
+		mark_reloc_symbols(ecp, sc);
+
+	/* Mark symbols used in section groups. */
+	if (ecp->v_grp == NULL)
+		mark_section_group_symbols(ecp, sc);
+
+	/*
+	 * Strip the symbol if specified by command line option -N,
+	 * unless it's used in relocation.
+	 */
+	if (lookup_symop_list(ecp, name, SYMOP_STRIP) != NULL) {
+		if (BIT_ISSET(ecp->v_rel, i)) {
+			warnx("not stripping symbol `%s' because it is named"
+			    " in a relocation", name);
+			return (0);
+		}
+		return (1);
+	}
 
 	if (is_needed_symbol(ecp, i, s))
 		return (0);
@@ -178,7 +237,12 @@ is_remove_symbol(struct elfcopy *ecp, size_t sc, int i, GElf_Sym *s,
 	if (ecp->strip == STRIP_UNNEEDED)
 		return (1);
 
-	if ((ecp->flags & DISCARD_LOCAL) && is_local_symbol(s->st_info))
+	if ((ecp->flags & DISCARD_LOCAL) && is_local_symbol(s->st_info) &&
+	    !is_debug_symbol(s->st_info))
+		return (1);
+
+	if ((ecp->flags & DISCARD_LLABEL) && is_local_symbol(s->st_info) &&
+	    !is_debug_symbol(s->st_info) && is_local_label(name))
 		return (1);
 
 	if (ecp->strip == STRIP_DEBUG && is_debug_symbol(s->st_info))
@@ -191,7 +255,7 @@ is_remove_symbol(struct elfcopy *ecp, size_t sc, int i, GElf_Sym *s,
  * Mark symbols refered by relocation entries.
  */
 static void
-mark_symbols(struct elfcopy *ecp, size_t sc)
+mark_reloc_symbols(struct elfcopy *ecp, size_t sc)
 {
 	const char	*name;
 	Elf_Data	*d;
@@ -269,6 +333,49 @@ mark_symbols(struct elfcopy *ecp, size_t sc)
 		    elf_errmsg(elferr));
 }
 
+static void
+mark_section_group_symbols(struct elfcopy *ecp, size_t sc)
+{
+	const char	*name;
+	Elf_Scn		*s;
+	GElf_Shdr	 sh;
+	size_t		 indx;
+	int		 elferr;
+
+	ecp->v_grp = calloc((sc + 7) / 8, 1);
+	if (ecp->v_grp == NULL)
+		err(EXIT_FAILURE, "calloc failed");
+
+	if (elf_getshstrndx(ecp->ein, &indx) == 0)
+		errx(EXIT_FAILURE, "elf_getshstrndx failed: %s",
+		    elf_errmsg(-1));
+
+	s = NULL;
+	while ((s = elf_nextscn(ecp->ein, s)) != NULL) {
+		if (gelf_getshdr(s, &sh) != &sh)
+			errx(EXIT_FAILURE, "elf_getshdr failed: %s",
+			    elf_errmsg(-1));
+
+		if (sh.sh_type != SHT_GROUP)
+			continue;
+
+		if ((name = elf_strptr(ecp->ein, indx, sh.sh_name)) == NULL)
+			errx(EXIT_FAILURE, "elf_strptr failed: %s",
+			    elf_errmsg(-1));
+		if (is_remove_section(ecp, name))
+			continue;
+
+		if (sh.sh_info > 0 && sh.sh_info < sc)
+			BIT_SET(ecp->v_grp, sh.sh_info);
+		else if (sh.sh_info != 0)
+			warnx("invalid symbox index");
+	}
+	elferr = elf_errno();
+	if (elferr != 0)
+		errx(EXIT_FAILURE, "elf_nextscn failed: %s",
+		    elf_errmsg(elferr));
+}
+
 static int
 generate_symbols(struct elfcopy *ecp)
 {
@@ -283,7 +390,7 @@ generate_symbols(struct elfcopy *ecp)
 	GElf_Sym	 sym;
 	Elf_Data*	 id;
 	Elf_Scn		*is;
-	size_t		 ishstrndx, namelen, ndx, nsyms, sc, symndx;
+	size_t		 ishstrndx, namelen, ndx, sc, symndx;
 	int		 ec, elferr, i;
 
 	if (elf_getshstrndx(ecp->ein, &ishstrndx) == 0)
@@ -299,17 +406,18 @@ generate_symbols(struct elfcopy *ecp)
 	if ((st_buf = calloc(1, sizeof(*st_buf))) == NULL)
 		err(EXIT_FAILURE, "calloc failed");
 	sy_buf->gcap = sy_buf->lcap = 64;
-	st_buf->gcap = 256;
-	st_buf->lcap = 64;
-	st_buf->lsz = 1;	/* '\0' at start. */
-	st_buf->gsz = 0;
-	nsyms = 0;
+	st_buf->g.cap = 256;
+	st_buf->l.cap = 64;
+	st_buf->l.sz = 1;	/* '\0' at start. */
+	st_buf->g.sz = 0;
 
 	ecp->symtab->sz = 0;
 	ecp->strtab->sz = 0;
 	ecp->symtab->buf = sy_buf;
 	ecp->strtab->buf = st_buf;
 
+	gsym = NULL;
+
 	/*
 	 * Create bit vector v_secsym, which is used to mark sections
 	 * that already have corresponding STT_SECTION symbols.
@@ -339,9 +447,12 @@ generate_symbols(struct elfcopy *ecp)
 	if (elferr != 0)
 		errx(EXIT_FAILURE, "elf_nextscn failed: %s",
 		    elf_errmsg(elferr));
-	/* FIXME don't panic if can't find .strtab */
-	if (symndx == 0)
-		errx(EXIT_FAILURE, "can't find .strtab section");
+
+	/* Symbol table should exist if this function is called. */
+	if (symndx == 0) {
+		warnx("can't find .strtab section");
+		goto clean;
+	}
 
 	/* Locate .symtab of input object. */
 	is = NULL;
@@ -369,7 +480,6 @@ generate_symbols(struct elfcopy *ecp)
 	 * output object, it is used by update_reloc() later to update
 	 * relocation information.
 	 */
-	gsym = NULL;
 	sc = ish.sh_size / ish.sh_entsize;
 	if (sc > 0) {
 		ecp->symndx = calloc(sc, sizeof(*ecp->symndx));
@@ -383,7 +493,7 @@ generate_symbols(struct elfcopy *ecp)
 			if (elferr != 0)
 				errx(EXIT_FAILURE, "elf_getdata failed: %s",
 				    elf_errmsg(elferr));
-			return (0);
+			goto clean;
 		}
 	} else
 		return (0);
@@ -423,6 +533,11 @@ generate_symbols(struct elfcopy *ecp)
 			    lookup_symop_list(ecp, name, SYMOP_KEEPG) == NULL)
 				sym.st_info = GELF_ST_INFO(STB_LOCAL,
 				    GELF_ST_TYPE(sym.st_info));
+			if (ecp->flags & LOCALIZE_HIDDEN &&
+			    sym.st_shndx != SHN_UNDEF &&
+			    is_hidden_symbol(sym.st_other))
+				sym.st_info = GELF_ST_INFO(STB_LOCAL,
+				    GELF_ST_TYPE(sym.st_info));
 		} else {
 			/* STB_LOCAL binding. */
 			if (lookup_symop_list(ecp, name, SYMOP_GLOBALIZE) !=
@@ -464,8 +579,11 @@ generate_symbols(struct elfcopy *ecp)
 		 * If the symbol is a STT_SECTION symbol, mark the section
 		 * it points to.
 		 */
-		if (GELF_ST_TYPE(sym.st_info) == STT_SECTION)
+		if (GELF_ST_TYPE(sym.st_info) == STT_SECTION &&
+		    sym.st_shndx < SHN_LORESERVE) {
+			assert(ecp->secndx[sym.st_shndx] < (uint64_t)ecp->nos);
 			BIT_SET(ecp->v_secsym, ecp->secndx[sym.st_shndx]);
+		}
 	}
 
 	/*
@@ -474,7 +592,7 @@ generate_symbols(struct elfcopy *ecp)
 	 * check if that only local symbol is the reserved symbol.
 	 */
 	if (sy_buf->nls <= 1 && sy_buf->ngs == 0)
-		return (0);
+		goto clean;
 
 	/*
 	 * Create STT_SECTION symbols for sections that do not already
@@ -501,6 +619,7 @@ generate_symbols(struct elfcopy *ecp)
 			sym.st_value = s->vma;
 			sym.st_size  = 0;
 			sym.st_info  = GELF_ST_INFO(STB_LOCAL, STT_SECTION);
+			sym.st_other = STV_DEFAULT;
 			/*
 			 * Don't let add_to_symtab() touch sym.st_shndx.
 			 * In this case, we know the index already.
@@ -522,10 +641,10 @@ generate_symbols(struct elfcopy *ecp)
 			/* Update st_name. */
 			if (ec == ELFCLASS32)
 				sy_buf->g32[ecp->symndx[i]].st_name +=
-				    st_buf->lsz;
+				    st_buf->l.sz;
 			else
 				sy_buf->g64[ecp->symndx[i]].st_name +=
-				    st_buf->lsz;
+				    st_buf->l.sz;
 
 			/* Update index map. */
 			ecp->symndx[i] += sy_buf->nls;
@@ -534,6 +653,12 @@ generate_symbols(struct elfcopy *ecp)
 	}
 
 	return (1);
+
+clean:
+	free(gsym);
+	free_symtab(ecp);
+
+	return (0);
 }
 
 void
@@ -575,7 +700,9 @@ create_symtab(struct elfcopy *ecp)
 	if (((ecp->flags & SYMTAB_INTACT) == 0) && !generate_symbols(ecp)) {
 		TAILQ_REMOVE(&ecp->v_sec, ecp->symtab, sec_list);
 		TAILQ_REMOVE(&ecp->v_sec, ecp->strtab, sec_list);
+		free(ecp->symtab->buf);
 		free(ecp->symtab);
+		free(ecp->strtab->buf);
 		free(ecp->strtab);
 		ecp->symtab = NULL;
 		ecp->strtab = NULL;
@@ -614,6 +741,8 @@ free_symtab(struct elfcopy *ecp)
 {
 	struct symbuf	*sy_buf;
 	struct strbuf	*st_buf;
+	struct sthash	*sh, *shtmp;
+	int i;
 
 	if (ecp->symtab != NULL && ecp->symtab->buf != NULL) {
 		sy_buf = ecp->symtab->buf;
@@ -629,10 +758,39 @@ free_symtab(struct elfcopy *ecp)
 
 	if (ecp->strtab != NULL && ecp->strtab->buf != NULL) {
 		st_buf = ecp->strtab->buf;
-		if (st_buf->l != NULL)
-			free(st_buf->l);
-		if (st_buf->g != NULL)
-			free(st_buf->g);
+		if (st_buf->l.buf != NULL)
+			free(st_buf->l.buf);
+		if (st_buf->g.buf != NULL)
+			free(st_buf->g.buf);
+		for (i = 0; i < STHASHSIZE; i++) {
+			LIST_FOREACH_SAFE(sh, &st_buf->l.hash[i], sh_next,
+			    shtmp) {
+				LIST_REMOVE(sh, sh_next);
+				free(sh);
+			}
+			LIST_FOREACH_SAFE(sh, &st_buf->g.hash[i], sh_next,
+			    shtmp) {
+				LIST_REMOVE(sh, sh_next);
+				free(sh);
+			}
+		}
+	}
+
+	if (ecp->symndx != NULL) {
+		free(ecp->symndx);
+		ecp->symndx = NULL;
+	}
+	if (ecp->v_rel != NULL) {
+		free(ecp->v_rel);
+		ecp->v_rel = NULL;
+	}
+	if (ecp->v_grp != NULL) {
+		free(ecp->v_grp);
+		ecp->v_grp = NULL;
+	}
+	if (ecp->v_secsym != NULL) {
+		free(ecp->v_secsym);
+		ecp->v_secsym = NULL;
 	}
 }
 
@@ -670,10 +828,10 @@ create_external_symtab(struct elfcopy *ecp)
 	if ((st_buf = calloc(1, sizeof(*st_buf))) == NULL)
 		err(EXIT_FAILURE, "calloc failed");
 	sy_buf->gcap = sy_buf->lcap = 64;
-	st_buf->gcap = 256;
-	st_buf->lcap = 64;
-	st_buf->lsz = 1;	/* '\0' at start. */
-	st_buf->gsz = 0;
+	st_buf->g.cap = 256;
+	st_buf->l.cap = 64;
+	st_buf->l.sz = 1;	/* '\0' at start. */
+	st_buf->g.sz = 0;
 
 	ecp->symtab->sz = 0;
 	ecp->strtab->sz = 0;
@@ -710,6 +868,8 @@ add_to_symtab(struct elfcopy *ecp, const char *name, uint64_t st_value,
 {
 	struct symbuf *sy_buf;
 	struct strbuf *st_buf;
+	struct sthash *sh;
+	uint32_t hash;
 	int pos;
 
 	/*
@@ -718,6 +878,8 @@ add_to_symtab(struct elfcopy *ecp, const char *name, uint64_t st_value,
 	 * It handles buffer growing, st_name calculating and st_shndx
 	 * updating for symbols with non-special section index.
 	 */
+#define	_ST_NAME_EMPTY_l 0
+#define	_ST_NAME_EMPTY_g -1
 #define	_ADDSYM(B, SZ) do {						\
 	if (sy_buf->B##SZ == NULL) {					\
 		sy_buf->B##SZ = malloc(sy_buf->B##cap *			\
@@ -742,35 +904,43 @@ add_to_symtab(struct elfcopy *ecp, const char *name, uint64_t st_value,
 	else								\
 		sy_buf->B##SZ[sy_buf->n##B##s].st_shndx	=		\
 			ecp->secndx[st_shndx];				\
-	if (st_buf->B == NULL) {					\
-		st_buf->B = calloc(st_buf->B##cap, sizeof(*st_buf->B));	\
-		if (st_buf->B == NULL)					\
+	if (st_buf->B.buf == NULL) {					\
+		st_buf->B.buf = calloc(st_buf->B.cap,			\
+		    sizeof(*st_buf->B.buf));				\
+		if (st_buf->B.buf == NULL)				\
 			err(EXIT_FAILURE, "malloc failed");		\
 	}								\
 	if (name != NULL && *name != '\0') {				\
-		pos = lookup_exact_string(st_buf->B,			\
-		    st_buf->B##sz, name);				\
+		pos = lookup_exact_string(st_buf->B.hash, st_buf->B.buf,\
+		    name);						\
 		if (pos != -1)						\
 			sy_buf->B##SZ[sy_buf->n##B##s].st_name = pos;	\
 		else {							\
 			sy_buf->B##SZ[sy_buf->n##B##s].st_name =	\
-			    st_buf->B##sz;				\
-			while (st_buf->B##sz + strlen(name) >=		\
-			    st_buf->B##cap - 1) {			\
-				st_buf->B##cap *= 2;			\
-				st_buf->B = realloc(st_buf->B,		\
-				    st_buf->B##cap);			\
-				if (st_buf->B == NULL)			\
+			    st_buf->B.sz;				\
+			while (st_buf->B.sz + strlen(name) >=		\
+			    st_buf->B.cap - 1) {			\
+				st_buf->B.cap *= 2;			\
+				st_buf->B.buf = realloc(st_buf->B.buf,	\
+				    st_buf->B.cap);			\
+				if (st_buf->B.buf == NULL)		\
 					err(EXIT_FAILURE,		\
 					    "realloc failed");		\
 			}						\
-			strncpy(&st_buf->B[st_buf->B##sz], name,	\
+			if ((sh = malloc(sizeof(*sh))) == NULL)		\
+				err(EXIT_FAILURE, "malloc failed");	\
+			sh->sh_off = st_buf->B.sz;			\
+			hash = str_hash(name);				\
+			LIST_INSERT_HEAD(&st_buf->B.hash[hash], sh,	\
+			    sh_next);					\
+			strncpy(&st_buf->B.buf[st_buf->B.sz], name,	\
 			    strlen(name));				\
-			st_buf->B[st_buf->B##sz + strlen(name)] = '\0';	\
-			st_buf->B##sz += strlen(name) + 1;		\
+			st_buf->B.buf[st_buf->B.sz + strlen(name)] = '\0'; \
+			st_buf->B.sz += strlen(name) + 1;		\
 		}							\
 	} else								\
-		sy_buf->B##SZ[sy_buf->n##B##s].st_name = 0;		\
+		sy_buf->B##SZ[sy_buf->n##B##s].st_name = 		\
+		    (Elf##SZ##_Word)_ST_NAME_EMPTY_##B;			\
 	sy_buf->n##B##s++;						\
 } while (0)
 
@@ -792,9 +962,11 @@ add_to_symtab(struct elfcopy *ecp, const char *name, uint64_t st_value,
 	/* Update section size. */
 	ecp->symtab->sz = (sy_buf->nls + sy_buf->ngs) *
 	    (ecp->oec == ELFCLASS32 ? sizeof(Elf32_Sym) : sizeof(Elf64_Sym));
-	ecp->strtab->sz = st_buf->lsz + st_buf->gsz;
+	ecp->strtab->sz = st_buf->l.sz + st_buf->g.sz;
 
 #undef	_ADDSYM
+#undef	_ST_NAME_EMPTY_l
+#undef	_ST_NAME_EMPTY_g
 }
 
 void
@@ -811,10 +983,17 @@ finalize_external_symtab(struct elfcopy *ecp)
 	sy_buf = ecp->symtab->buf;
 	st_buf = ecp->strtab->buf;
 	for (i = 0; (size_t) i < sy_buf->ngs; i++) {
-		if (ecp->oec == ELFCLASS32)
-			sy_buf->g32[i].st_name += st_buf->lsz;
-		else
-			sy_buf->g64[i].st_name += st_buf->lsz;
+		if (ecp->oec == ELFCLASS32) {
+			if (sy_buf->g32[i].st_name == (Elf32_Word)-1)
+				sy_buf->g32[i].st_name = 0;
+			else
+				sy_buf->g32[i].st_name += st_buf->l.sz;
+		} else {
+			if (sy_buf->g64[i].st_name == (Elf64_Word)-1)
+				sy_buf->g64[i].st_name = 0;
+			else
+				sy_buf->g64[i].st_name += st_buf->l.sz;
+		}
 	}
 }
 
@@ -901,19 +1080,19 @@ create_symtab_data(struct elfcopy *ecp)
 		    elf_errmsg(-1));
 	lstdata->d_align	= 1;
 	lstdata->d_off		= 0;
-	lstdata->d_buf		= st_buf->l;
-	lstdata->d_size		= st_buf->lsz;
+	lstdata->d_buf		= st_buf->l.buf;
+	lstdata->d_size		= st_buf->l.sz;
 	lstdata->d_type		= ELF_T_BYTE;
 	lstdata->d_version	= EV_CURRENT;
 
-	if (st_buf->gsz > 0) {
+	if (st_buf->g.sz > 0) {
 		if ((gstdata = elf_newdata(st->os)) == NULL)
 			errx(EXIT_FAILURE, "elf_newdata() failed: %s.",
 			    elf_errmsg(-1));
 		gstdata->d_align	= 1;
 		gstdata->d_off		= lstdata->d_size;
-		gstdata->d_buf		= st_buf->g;
-		gstdata->d_size		= st_buf->gsz;
+		gstdata->d_buf		= st_buf->g.buf;
+		gstdata->d_size		= st_buf->g.sz;
 		gstdata->d_type		= ELF_T_BYTE;
 		gstdata->d_version	= EV_CURRENT;
 	}
@@ -955,66 +1134,72 @@ add_to_symop_list(struct elfcopy *ecp, const char *name, const char *newname,
 {
 	struct symop *s;
 
-	if ((s = lookup_symop_list(ecp, name, ~0U)) == NULL) {
-		if ((s = calloc(1, sizeof(*s))) == NULL)
-			errx(EXIT_FAILURE, "not enough memory");
-		s->name = name;
-		if (op == SYMOP_REDEF)
-			s->newname = newname;
-	}
+	assert (name != NULL);
+	STAILQ_FOREACH(s, &ecp->v_symop, symop_list)
+		if (!strcmp(name, s->name))
+			goto found;
 
-	s->op |= op;
+	if ((s = calloc(1, sizeof(*s))) == NULL)
+		errx(EXIT_FAILURE, "not enough memory");
 	STAILQ_INSERT_TAIL(&ecp->v_symop, s, symop_list);
-}
-
-static int
-match_wildcard(const char *name, const char *pattern)
-{
-	int reverse, match;
-
-	reverse = 0;
-	if (*pattern == '!') {
-		reverse = 1;
-		pattern++;
-	}
-
-	match = 0;
-	if (!fnmatch(pattern, name, 0)) {
-		match = 1;
-		printf("string '%s' match to pattern '%s'\n", name, pattern);
-	}
-
-	return (reverse ? !match : match);
+	s->name = name;
+found:
+	if (op == SYMOP_REDEF)
+		s->newname = newname;
+	s->op |= op;
 }
 
 struct symop *
 lookup_symop_list(struct elfcopy *ecp, const char *name, unsigned int op)
 {
-	struct symop *s;
+	struct symop *s, *ret;
+	const char *pattern;
 
 	STAILQ_FOREACH(s, &ecp->v_symop, symop_list) {
-		if (name == NULL || !strcmp(name, s->name) ||
-		    ((ecp->flags & WILDCARD) && match_wildcard(name, s->name)))
-			if ((s->op & op) != 0)
+		if ((s->op & op) == 0)
+			continue;
+		if (name == NULL || !strcmp(name, s->name))
 				return (s);
+		if ((ecp->flags & WILDCARD) == 0)
+			continue;
+
+		/* Handle wildcards. */
+		pattern = s->name;
+		if (pattern[0] == '!') {
+			/* Negative match. */
+			pattern++;
+			ret = NULL;
+		} else {
+			/* Regular wildcard match. */
+			ret = s;
+		}
+		if (!fnmatch(pattern, name, 0))
+			return (ret);
 	}
 
 	return (NULL);
 }
 
 static int
-lookup_exact_string(const char *buf, size_t sz, const char *s)
+lookup_exact_string(hash_head *buckets, const char *buf, const char *s)
 {
-	const char	*b;
-	size_t		 slen;
-
-	slen = strlen(s);
-	for (b = buf; b < buf + sz; b += strlen(b) + 1) {
-		if (strlen(b) != slen)
-			continue;
-		if (!strcmp(b, s))
-			return (b - buf);
-	}
+	struct sthash	*sh;
+	uint32_t	 hash;
 
+	hash = str_hash(s);
+	LIST_FOREACH(sh, &buckets[hash], sh_next)
+		if (strcmp(buf + sh->sh_off, s) == 0)
+			return sh->sh_off;
 	return (-1);
 }
+
+uint32_t
+str_hash(const char *s)
+{
+	uint32_t hash;
+
+	for (hash = 2166136261UL; *s; s++)
+		hash = (hash ^ *s) * 16777619;
+
+	return (hash & (STHASHSIZE - 1));
+}
