@@ -26,7 +26,7 @@
 
 #include "_libdwarf.h"
 
-ELFTC_VCSID("$Id: libdwarf_lineno.c 2596 2012-09-18 14:26:04Z jkoshy $");
+ELFTC_VCSID("$Id: libdwarf_lineno.c 3164 2015-02-19 01:20:12Z kaiwang27 $");
 
 static int
 _dwarf_lineno_add_file(Dwarf_LineInfo li, uint8_t **p, const char *compdir,
@@ -87,9 +87,8 @@ _dwarf_lineno_run_program(Dwarf_CU cu, Dwarf_LineInfo li, uint8_t *p,
 {
 	Dwarf_Debug dbg;
 	Dwarf_Line ln, tln;
-	uint64_t address, file, line, column, isa, opsize;
+	uint64_t address, file, line, column, opsize;
 	int is_stmt, basic_block, end_sequence;
-	int prologue_end, epilogue_begin;
 	int ret;
 
 #define	RESET_REGISTERS						\
@@ -101,8 +100,6 @@ _dwarf_lineno_run_program(Dwarf_CU cu, Dwarf_LineInfo li, uint8_t *p,
 		is_stmt	       = li->li_defstmt;		\
 		basic_block    = 0;				\
 		end_sequence   = 0;				\
-		prologue_end   = 0;				\
-		epilogue_begin = 0;				\
 	} while(0)
 
 #define	APPEND_ROW						\
@@ -152,6 +149,7 @@ _dwarf_lineno_run_program(Dwarf_CU cu, Dwarf_LineInfo li, uint8_t *p,
 			case DW_LNE_end_sequence:
 				p++;
 				end_sequence = 1;
+				APPEND_ROW;
 				RESET_REGISTERS;
 				break;
 			case DW_LNE_set_address:
@@ -180,8 +178,6 @@ _dwarf_lineno_run_program(Dwarf_CU cu, Dwarf_LineInfo li, uint8_t *p,
 			case DW_LNS_copy:
 				APPEND_ROW;
 				basic_block = 0;
-				prologue_end = 0;
-				epilogue_begin = 0;
 				break;
 			case DW_LNS_advance_pc:
 				address += _dwarf_decode_uleb128(&p) *
@@ -209,13 +205,11 @@ _dwarf_lineno_run_program(Dwarf_CU cu, Dwarf_LineInfo li, uint8_t *p,
 				address += dbg->decode(&p, 2);
 				break;
 			case DW_LNS_set_prologue_end:
-				prologue_end = 1;
 				break;
 			case DW_LNS_set_epilogue_begin:
-				epilogue_begin = 1;
 				break;
 			case DW_LNS_set_isa:
-				isa = _dwarf_decode_uleb128(&p);
+				(void) _dwarf_decode_uleb128(&p);
 				break;
 			default:
 				/* Unrecognized extened opcodes. What to do? */
@@ -232,8 +226,6 @@ _dwarf_lineno_run_program(Dwarf_CU cu, Dwarf_LineInfo li, uint8_t *p,
 			address += ADDRESS(*p);
 			APPEND_ROW;
 			basic_block = 0;
-			prologue_end = 0;
-			epilogue_begin = 0;
 			p++;
 		}
 	}
@@ -323,6 +315,8 @@ _dwarf_lineno_init(Dwarf_Die die, uint64_t offset, Dwarf_Error *error)
 	li->li_hdrlen = dbg->read(ds->ds_data, &offset, dwarf_size);
 	hdroff = offset;
 	li->li_minlen = dbg->read(ds->ds_data, &offset, 1);
+	if (li->li_version == 4)
+		li->li_maxop = dbg->read(ds->ds_data, &offset, 1);
 	li->li_defstmt = dbg->read(ds->ds_data, &offset, 1);
 	li->li_lbase = dbg->read(ds->ds_data, &offset, 1);
 	li->li_lrange = dbg->read(ds->ds_data, &offset, 1);
@@ -368,10 +362,13 @@ _dwarf_lineno_init(Dwarf_Die die, uint64_t offset, Dwarf_Error *error)
 		goto fail_cleanup;
 	}
 
-	if ((li->li_incdirs = malloc(length * sizeof(char *))) == NULL) {
-		ret = DW_DLE_MEMORY;
-		DWARF_SET_ERROR(dbg, error, ret);
-		goto fail_cleanup;
+	if (length != 0) {
+		if ((li->li_incdirs = malloc(length * sizeof(char *))) ==
+		    NULL) {
+			ret = DW_DLE_MEMORY;
+			DWARF_SET_ERROR(dbg, error, ret);
+			goto fail_cleanup;
+		}
 	}
 
 	/* Fill in include dir array. */
@@ -478,7 +475,8 @@ _dwarf_lineno_gen_program(Dwarf_P_Debug dbg, Dwarf_P_Section ds,
 	Dwarf_Unsigned address, file, line, spc;
 	Dwarf_Unsigned addr0, maddr;
 	Dwarf_Signed line0, column;
-	int is_stmt, basic_block, end_sequence;
+	int is_stmt, basic_block;
+	int need_copy;
 	int ret;
 
 #define	RESET_REGISTERS						\
@@ -489,7 +487,6 @@ _dwarf_lineno_gen_program(Dwarf_P_Debug dbg, Dwarf_P_Section ds,
 		column	       = 0;				\
 		is_stmt	       = li->li_defstmt;		\
 		basic_block    = 0;				\
-		end_sequence   = 0;				\
 	} while(0)
 
 	li = dbg->dbgp_lineinfo;
@@ -509,8 +506,15 @@ _dwarf_lineno_gen_program(Dwarf_P_Debug dbg, Dwarf_P_Section ds,
 			    dwarf_drt_data_reloc, dbg->dbg_pointer_size,
 			    ds->ds_size, ln->ln_symndx, ln->ln_addr,
 			    NULL, error));
-			goto next_line;
+			address = ln->ln_addr;
+			continue;
 		} else if (ln->ln_endseq) {
+			addr0 = (ln->ln_addr - address) / li->li_minlen;
+			if (addr0 != 0) {
+				RCHECK(WRITE_VALUE(DW_LNS_advance_pc, 1));
+				RCHECK(WRITE_ULEB128(addr0));
+			}
+
 			/*
 			 * Generate DW_LNE_end_sequence.
 			 */
@@ -550,49 +554,53 @@ _dwarf_lineno_gen_program(Dwarf_P_Debug dbg, Dwarf_P_Section ds,
 		addr0 = (ln->ln_addr - address) / li->li_minlen;
 		line0 = ln->ln_lineno - line;
 
+		if (addr0 == 0 && line0 == 0)
+			continue;
+
 		/*
-		 * Generate DW_LNS_advance_line if line delta is out of
-		 * range.
+		 * Check if line delta is with the range and if the special
+		 * opcode can be used.
 		 */
 		assert(li->li_lbase <= 0);
-		if (line0 < li->li_lbase ||
-		    line0 > li->li_lbase + li->li_lrange - 1) {
+		if (line0 >= li->li_lbase &&
+		    line0 <= li->li_lbase + li->li_lrange - 1) {
+			spc = (line0 - li->li_lbase) +
+			    (li->li_lrange * addr0) + li->li_opbase;
+			if (spc <= 255) {
+				RCHECK(WRITE_VALUE(spc, 1));
+				basic_block = 0;
+				goto next_line;
+			}
+		}
+
+		/* Generate DW_LNS_advance_line for line number change. */
+		if (line0 != 0) {
 			RCHECK(WRITE_VALUE(DW_LNS_advance_line, 1));
 			RCHECK(WRITE_SLEB128(line0));
 			line0 = 0;
+			need_copy = 1;
+		} else
+			need_copy = basic_block;
+
+		if (addr0 != 0) {
+			/* See if it can be handled by DW_LNS_const_add_pc. */
+			spc = (line0 - li->li_lbase) +
+			    (li->li_lrange * (addr0 - maddr)) + li->li_opbase;
+			if (addr0 >= maddr && spc <= 255) {
+				RCHECK(WRITE_VALUE(DW_LNS_const_add_pc, 1));
+				RCHECK(WRITE_VALUE(spc, 1));
+			} else {
+				/* Otherwise we use DW_LNS_advance_pc. */
+				RCHECK(WRITE_VALUE(DW_LNS_advance_pc, 1));
+				RCHECK(WRITE_ULEB128(addr0));
+			}
 		}
 
-		/*
-		 * Check if it can be fit in the special ops.
-		 */
-		spc = (line0 - li->li_lbase) + (li->li_lrange * addr0) +
-		    li->li_opbase;
-		if (spc <= 255) {
-			RCHECK(WRITE_VALUE(spc, 1));
+		if (need_copy) {
+			RCHECK(WRITE_VALUE(DW_LNS_copy, 1));
 			basic_block = 0;
-			goto next_line;
 		}
 
-		/*
-		 * See if this can be handled by DW_LNS_const_add_pc.
-		 */
-		spc = (line0 - li->li_lbase) +
-		    (li->li_lrange * (addr0 - maddr)) + li->li_opbase;
-		if (spc <= 255) {
-			RCHECK(WRITE_VALUE(DW_LNS_const_add_pc, 1));
-			RCHECK(WRITE_VALUE(spc, 1));
-			basic_block = 0;
-			goto next_line;
-		}
-
-		/*
-		 * Otherwise we have to use DW_LNS_advance_pc.
-		 */
-		RCHECK(WRITE_VALUE(DW_LNS_advance_pc, 1));
-		RCHECK(WRITE_ULEB128(addr0));
-		RCHECK(WRITE_VALUE(DW_LNS_copy, 1));
-		basic_block = 0;
-
 	next_line:
 		address = ln->ln_addr;
 		line = ln->ln_lineno;
@@ -613,6 +621,8 @@ _dwarf_get_minlen(Dwarf_P_Debug dbg)
 	assert(dbg != NULL);
 
 	switch (dbg->dbgp_isa) {
+	case DW_ISA_ARM:
+		return (2);
 	case DW_ISA_X86:
 	case DW_ISA_X86_64:
 		return (1);
@@ -668,7 +678,7 @@ _dwarf_lineno_gen(Dwarf_P_Debug dbg, Dwarf_Error *error)
 	offset = ds->ds_size;
 	RCHECK(WRITE_VALUE(li->li_hdrlen, 4));
 
-	/* Write minimum instruction length. FIXME should be arch-specific. */
+	/* Write minimum instruction length. */
 	RCHECK(WRITE_VALUE(li->li_minlen, 1));
 
 	/*
