@@ -24,15 +24,14 @@
  * SUCH DAMAGE.
  */
 
-#include <sys/cdefs.h>
-
 #include <assert.h>
 #include <gelf.h>
 #include <limits.h>
+#include <stdint.h>
 
 #include "_libelf.h"
 
-ELFTC_VCSID("$Id: gelf_rela.c 2272 2011-12-03 17:07:31Z jkoshy $");
+ELFTC_VCSID("$Id: gelf_rela.c 3177 2015-03-30 18:19:41Z emaste $");
 
 GElf_Rela *
 gelf_getrela(Elf_Data *ed, int ndx, GElf_Rela *dst)
@@ -71,8 +70,9 @@ gelf_getrela(Elf_Data *ed, int ndx, GElf_Rela *dst)
 	msz = _libelf_msize(ELF_T_RELA, ec, e->e_version);
 
 	assert(msz > 0);
+	assert(ndx >= 0);
 
-	if (msz * ndx >= d->d_data.d_size) {
+	if (msz * (size_t) ndx >= d->d_data.d_size) {
 		LIBELF_SET_ERROR(ARGUMENT, 0);
 		return (NULL);
 	}
@@ -131,9 +131,11 @@ gelf_update_rela(Elf_Data *ed, int ndx, GElf_Rela *dr)
 	}
 
 	msz = _libelf_msize(ELF_T_RELA, ec, e->e_version);
-	assert(msz > 0);
 
-	if (msz * ndx >= d->d_data.d_size) {
+	assert(msz > 0);
+	assert(ndx >= 0);
+
+	if (msz * (size_t) ndx >= d->d_data.d_size) {
 		LIBELF_SET_ERROR(ARGUMENT, 0);
 		return (0);
 	}
@@ -148,8 +150,9 @@ gelf_update_rela(Elf_Data *ed, int ndx, GElf_Rela *dr)
 			LIBELF_SET_ERROR(RANGE, 0);
 			return (0);
 		}
-		rela32->r_info = ELF32_R_INFO(ELF64_R_SYM(dr->r_info),
-		    ELF64_R_TYPE(dr->r_info));
+		rela32->r_info = ELF32_R_INFO(
+			(Elf32_Word) ELF64_R_SYM(dr->r_info),
+			(Elf32_Word) ELF64_R_TYPE(dr->r_info));
 
 		LIBELF_COPY_S32(rela32, dr, r_addend);
 	} else {
