@@ -24,19 +24,17 @@
  * SUCH DAMAGE.
  */
 
-#include <sys/cdefs.h>
-
 #include <sys/types.h>
 
 #include <libelf.h>
 
 #include "_libelf.h"
 
-ELFTC_VCSID("$Id: libelf_align.c 2225 2011-11-26 18:55:54Z jkoshy $");
+ELFTC_VCSID("$Id: libelf_align.c 3174 2015-03-27 17:13:41Z emaste $");
 
 struct align {
-	int a32;
-	int a64;
+	unsigned int a32;
+	unsigned int a64;
 };
 
 #ifdef	__GNUC__
@@ -87,7 +85,7 @@ static struct align malign[ELF_T_NUM] = {
 	[ELF_T_GNUHASH] = MALIGN_WORD()
 };
 
-int
+unsigned int
 _libelf_malign(Elf_Type t, int elfclass)
 {
 	if (t >= ELF_T_NUM || (int) t < 0)
@@ -126,7 +124,7 @@ static struct align falign[ELF_T_NUM] = {
 	[ELF_T_GNUHASH] = FALIGN(4,8)
 };
 
-int
+unsigned int
 _libelf_falign(Elf_Type t, int elfclass)
 {
 	if (t >= ELF_T_NUM || (int) t < 0)
