@@ -24,7 +24,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: _libelftc.h 2251 2011-11-30 16:50:06Z jkoshy $
+ * $Id: _libelftc.h 3174 2015-03-27 17:13:41Z emaste $
  */
 
 #ifndef	__LIBELFTC_H_
@@ -56,24 +56,38 @@ struct vector_str {
 };
 
 #define BUFFER_GROWFACTOR	1.618
-#define VECTOR_DEF_CAPACITY	8
-#define	ELFTC_ISDIGIT(C) 	(isdigit((C) & 0xFF))
 
-void	vector_str_dest(struct vector_str *);
-int	vector_str_find(const struct vector_str *v, const char *o, size_t l);
-char	*vector_str_get_flat(const struct vector_str *v, size_t *l);
-bool	vector_str_init(struct vector_str *);
-bool	vector_str_pop(struct vector_str *);
-bool	vector_str_push(struct vector_str *, const char *, size_t);
-bool	vector_str_push_vector_head(struct vector_str *dst,
-	    struct vector_str *org);
-char	*vector_str_substr(const struct vector_str *v, size_t begin, size_t end,
-	    size_t *r_len);
-char	*cpp_demangle_gnu3(const char *);
-bool	is_cpp_mangled_gnu3(const char *);
-char	*cpp_demangle_gnu2(const char *);
-bool	is_cpp_mangled_gnu2(const char *);
-char	*cpp_demangle_ARM(const char *);
-bool	is_cpp_mangled_ARM(const char *);
+#define	ELFTC_FAILURE		0
+#define	ELFTC_ISDIGIT(C) 	(isdigit((C) & 0xFF))
+#define	ELFTC_SUCCESS		1
+
+#define VECTOR_DEF_CAPACITY	8
+
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+char	*cpp_demangle_ARM(const char *_org);
+char	*cpp_demangle_gnu2(const char *_org);
+char	*cpp_demangle_gnu3(const char *_org);
+bool	is_cpp_mangled_ARM(const char *_org);
+bool	is_cpp_mangled_gnu2(const char *_org);
+bool	is_cpp_mangled_gnu3(const char *_org);
+unsigned int	libelftc_hash_string(const char *);
+void	vector_str_dest(struct vector_str *_vec);
+int	vector_str_find(const struct vector_str *_vs, const char *_str,
+    size_t _len);
+char	*vector_str_get_flat(const struct vector_str *_vs, size_t *_len);
+bool	vector_str_init(struct vector_str *_vs);
+bool	vector_str_pop(struct vector_str *_vs);
+bool	vector_str_push(struct vector_str *_vs, const char *_str,
+    size_t _len);
+bool	vector_str_push_vector_head(struct vector_str *_dst,
+    struct vector_str *_org);
+char	*vector_str_substr(const struct vector_str *_vs, size_t _begin,
+    size_t _end, size_t *_rlen);
+#ifdef __cplusplus
+}
+#endif
 
 #endif	/* __LIBELFTC_H */
