@@ -48,7 +48,7 @@
 
 #include "_elftc.h"
 
-ELFTC_VCSID("$Id: nm.c 2484 2012-04-07 15:52:05Z kaiwang27 $");
+ELFTC_VCSID("$Id: nm.c 3179 2015-03-31 19:38:56Z emaste $");
 
 /* symbol information list */
 STAILQ_HEAD(sym_head, sym_entry);
@@ -65,7 +65,7 @@ typedef void (*fn_sym_print)(const GElf_Sym *);
 typedef int (*fn_filter)(char, const GElf_Sym *, const char *);
 
 /* output filter list */
-SLIST_HEAD(filter_head, filter_entry) nm_out_filter =
+static SLIST_HEAD(filter_head, filter_entry) nm_out_filter =
     SLIST_HEAD_INITIALIZER(nm_out_filter);
 
 struct filter_entry {
@@ -248,6 +248,7 @@ static const struct option nm_longopts[] = {
 	{ "defined-only",	no_argument,		&nm_opts.def_only, 1},
 	{ "demangle",		optional_argument,	NULL,		'C' },
 	{ "dynamic",		no_argument,		NULL,		'D' },
+	{ "extern-only",	no_argument,		NULL,		'g' },
 	{ "format",		required_argument,	NULL,		'F' },
 	{ "help",		no_argument,		NULL,		'h' },
 	{ "line-numbers",	no_argument,		NULL,		'l' },
@@ -378,7 +379,7 @@ cmp_value(const void *lp, const void *rp)
 		/* Both defined */
 		if (l->sym->st_value == r->sym->st_value)
 			return (strcmp(l->name, r->name));
-		return (l->sym->st_value - r->sym->st_value);
+		return (l->sym->st_value > r->sym->st_value ? 1 : -1);
 	case 1:
 		/* One undefined */
 		return (l_is_undef == 0 ? 1 : -1);
@@ -947,6 +948,36 @@ get_block_value(Dwarf_Debug dbg, Dwarf_Block *block)
 	return (0);
 }
 
+static char *
+find_object_name(Dwarf_Debug dbg, Dwarf_Die die)
+{
+	Dwarf_Die ret_die;
+	Dwarf_Attribute at;
+	Dwarf_Off off;
+	Dwarf_Error de;
+	const char *str;
+	char *name;
+
+	if (dwarf_attrval_string(die, DW_AT_name, &str, &de) == DW_DLV_OK) {
+		if ((name = strdup(str)) == NULL) {
+			warn("strdup");
+			return (NULL);
+		}
+		return (name);
+	}
+
+	if (dwarf_attr(die, DW_AT_specification, &at, &de) != DW_DLV_OK)
+		return (NULL);
+
+	if (dwarf_global_formref(at, &off, &de) != DW_DLV_OK)
+		return (NULL);
+
+	if (dwarf_offdie(dbg, off, &ret_die, &de) != DW_DLV_OK)
+		return (NULL);
+
+	return (find_object_name(dbg, ret_die));
+}
+
 static void
 search_line_attr(Dwarf_Debug dbg, struct func_info_head *func_info,
     struct var_info_head *var_info, Dwarf_Die die, char **src_files,
@@ -961,7 +992,6 @@ search_line_attr(Dwarf_Debug dbg, struct func_info_head *func_info,
 	Dwarf_Error de;
 	struct func_info_entry *func;
 	struct var_info_entry *var;
-	const char *str;
 	int ret;
 
 	if (dwarf_tag(die, &tag, &de) != DW_DLV_OK) {
@@ -1011,16 +1041,12 @@ search_line_attr(Dwarf_Debug dbg, struct func_info_head *func_info,
 		    DW_DLV_OK)
 			var->line = udata;
 
-		if (dwarf_attrval_string(die, DW_AT_name, &str, &de) ==
-		    DW_DLV_OK) {
-			var->name = strdup(str);
-			if (var->name == NULL) {
-				warn("strdup");
-				if (var->file)
-					free(var->file);
-				free(var);
-				goto cont_search;
-			}
+		var->name = find_object_name(dbg, die);
+		if (var->name == NULL) {
+			if (var->file)
+				free(var->file);
+			free(var);
+			goto cont_search;
 		}
 
 		if (dwarf_attr(die, DW_AT_location, &at, &de) == DW_DLV_OK &&
@@ -1063,16 +1089,12 @@ search_line_attr(Dwarf_Debug dbg, struct func_info_head *func_info,
 		    DW_DLV_OK)
 			func->line = udata;
 
-		if (dwarf_attrval_string(die, DW_AT_name, &str, &de) ==
-		    DW_DLV_OK) {
-			func->name = strdup(str);
-			if (func->name == NULL) {
-				warn("strdup");
-				if (func->file)
-					free(func->file);
-				free(func);
-				goto cont_search;
-			}
+		func->name = find_object_name(dbg, die);
+		if (func->name == NULL) {
+			if (func->file)
+				free(func->file);
+			free(func);
+			goto cont_search;
 		}
 
 		if (dwarf_attrval_unsigned(die, DW_AT_low_pc, &udata, &de) ==
@@ -1524,7 +1546,8 @@ print_lineno(struct sym_entry *ep, struct func_info_head *func_info,
 	/* For function symbol, search the function line information list.  */
 	if ((ep->sym->st_info & 0xf) == STT_FUNC && func_info != NULL) {
 		SLIST_FOREACH(func, func_info, entries) {
-			if (!strcmp(ep->name, func->name) &&
+			if (func->name != NULL &&
+			    !strcmp(ep->name, func->name) &&
 			    ep->sym->st_value >= func->lowpc &&
 			    ep->sym->st_value < func->highpc) {
 				printf("\t%s:%" PRIu64, func->file, func->line);
@@ -2042,7 +2065,7 @@ usage(int exitcode)
 \n  -f                        Produce full output (default).\
 \n      --format=format       Display output in specific format.  Allowed\
 \n                            formats are: \"bsd\", \"posix\" and \"sysv\".\
-\n  -g                        Display only global symbol information.\
+\n  -g, --extern-only         Display only global symbol information.\
 \n  -h, --help                Show this help message.\
 \n  -l, --line-numbers        Display filename and linenumber using\
 \n                            debugging information.\
