@@ -24,7 +24,6 @@
  * SUCH DAMAGE.
  */
 
-#include <sys/cdefs.h>
 #include <assert.h>
 #include <err.h>
 #include <fcntl.h>
@@ -39,7 +38,7 @@
 
 #include "_elftc.h"
 
-ELFTC_VCSID("$Id: size.c 2350 2011-12-19 10:20:06Z jkoshy $");
+ELFTC_VCSID("$Id: size.c 3242 2015-08-07 12:47:11Z emaste $");
 
 #define	BUF_SIZE			1024
 #define	ELF_ALIGN(val,x) (((val)+(x)-1) & ~((x)-1))
@@ -441,7 +440,6 @@ handle_phdr(Elf *elf, GElf_Ehdr *elfhdr, GElf_Phdr *phdr,
 	if (elf == NULL || elfhdr == NULL || phdr == NULL)
 		return;
 
-	size = addr = 0;
 	split = (phdr->p_memsz > 0) && 	(phdr->p_filesz > 0) &&
 	    (phdr->p_memsz > phdr->p_filesz);
 
@@ -601,7 +599,7 @@ handle_elf(char const *name)
 			    arhdr->ar_name);
 			continue;
 		}
-		/* Core dumps are handled seperately */
+		/* Core dumps are handled separately */
 		if (elfhdr.e_shnum == 0 && elfhdr.e_type == ET_CORE) {
 			exit_code = handle_core(name, elf, &elfhdr);
 			(void) elf_end(elf);
