@@ -24,7 +24,6 @@
  * SUCH DAMAGE.
  */
 
-#include <sys/cdefs.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 
@@ -38,6 +37,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sysexits.h>
 #include <unistd.h>
 
 #include <libelf.h>
@@ -46,7 +46,7 @@
 
 #include "_elftc.h"
 
-ELFTC_VCSID("$Id: strings.c 2351 2011-12-19 11:20:37Z jkoshy $");
+ELFTC_VCSID("$Id: strings.c 3360 2016-01-24 18:34:06Z jkoshy $");
 
 enum return_code {
 	RETURN_OK,
@@ -74,10 +74,10 @@ enum encoding_style {
 	  ((c) == '\t' || isprint((c)) ||			\
 	      (encoding == ENCODING_8BIT && (c) > 127)))
 
-
-int encoding_size, entire_file, min_len, show_filename, show_loc;
-enum encoding_style encoding;
-enum radix_style radix;
+static int encoding_size, entire_file, show_filename, show_loc;
+static enum encoding_style encoding;
+static enum radix_style radix;
+static intmax_t min_len;
 
 static struct option strings_longopts[] = {
 	{ "all",		no_argument,		NULL,	'a'},
@@ -145,7 +145,10 @@ main(int argc, char **argv)
 			show_filename = 1;
 			break;
 		case 'n':
-			min_len = (int)strtoimax(optarg, (char**)NULL, 10);
+			min_len = strtoimax(optarg, (char**)NULL, 10);
+			if (min_len <= 0)
+				errx(EX_USAGE, "option -n should specify a "
+				    "positive decimal integer.");
 			break;
 		case 'o':
 			show_loc = 1;
